# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Any


class IRequestPrincipalIntrospecter:
    """Knows how to obtain details about a specific :class:`IRequestPrincipal`
    implementation.
    """
    __module__: str

    async def introspect(self, principal: Any) -> Any:
        raise NotImplementedError